#!/usr/bin/python

import sys
import os
import os.path

def is_in_path(cmd):
    for directory in os.environ['PATH'].split(os.path.pathsep):
        if os.access(directory+"/"+cmd,os.X_OK):
            return True
    return False

class FoldsIterator:
    def __init__(self,folds):
        self.folds=folds
        self.current=0
    def checkPosition(self):
        if self.current<0 or self.current>=len(self.folds):
            raise StopIteration
    def __iter__(self):
        return self
    def next(self):
        self.checkPosition()    
        self.current+=1    
        return self.folds[self.current-1]

class Folding:
    def __init__(self,command):
        self.folds=[]
        if type(command)!=list:
            command=[command]
        for c in command:
            if (is_in_path(c)==False):
                print >>sys.stderr,c+" not available: install it and/or fixe your path"
                sys.exit(2)
    def __iter__(self):
        return FoldsIterator(self.folds)

    def fold(self,sequence,options=""):
        self.folds=[] # forget old folds (still accessible via an iterator)
        try:
            os.mkdir(".fold")
        except:
            pass
        os.chdir(".fold")
        self.doFold(sequence,options)
        for root, dirs, files in os.walk(".", topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
            for name in dirs:
                os.rmdir(os.path.join(root, name))
        os.chdir(os.pardir)
        return self.__iter__()
    
    def doFold(self,sequence,options):
        raise "doFold must be overloaded"
        

class RNAsubopt(Folding):
    def __init__(self):
        Folding.__init__(self,"RNAsubopt")
    def doFold(self,sequence,options):        
        f=os.popen("echo "+sequence+" | RNAsubopt "+options)
        folds=f.readlines()
        if f.close()!=None:
            print >>sys.stderr,"RNAsubopt error!"
            return        
        for f in folds[1:]:
            f=f.strip()
            if f.find(" ")!=-1:
                f=f[:f.find(" ")]            
            self.folds.append((">\n",sequence+"\n",f))
            

class RNAfold(Folding):
    def __init__(self):
        Folding.__init__(self,"RNAfold")
    def doFold(self,sequence,options):        
        f=os.popen("echo "+sequence+" | RNAfold "+options)
        folds=f.readlines()
        if f.close()!=None:
            print >>sys.stderr,"RNAfold error!"
            return        
        for f in folds[1:]:
            f=f.strip()
            if f.find(" ")!=-1:
                f=f[:f.find(" ")]
            self.folds.append((">\n",sequence+"\n",f))
            


class RNAshapes(Folding):
    def __init__(self):
        Folding.__init__(self,"RNAshapes")
        
    def doFold(self,sequence,options):
        f=os.popen("echo "+sequence+" | RNAshapes "+options)
        folds=f.readlines()
        if f.close()!=None:
            print >>sys.stderr,"RNAsubopt error!"
            return
        del folds[0] # sequence
        for f in folds:
            f=f[f.find(" "):] # remove energie
            f=f.lstrip() # remove blank
            self.folds.append((">\n",sequence+"\n",f[:f.find(" ")]+"\n"))
        
    
    

class MFold(Folding):
    def __init__(self):
        Folding.__init__(self,["mfold","rnaconverter"])
    def doFold(self,sequence,options):
        try:
            os.remove("seq.sav")
        except:
            pass
        print >>file("seq","w"),sequence
        r=os.popen("mfold SEQ=seq "+options+" 2>&1 > /dev/null")
        s=r.read()
        if r.close()!=None:
            print >>sys.stderr,"MFold error!","mfold SEQ=seq ("+sequence+")"+options
            print >>sys.stderr,"perhaps there is no foldings!"
            return
        files=os.listdir(os.curdir)
        for f in files:
            if f.startswith("seq_") and f.endswith(".ct"):
                os.system("rnaconverter -o ss -d "+f+" 2>&1 > /dev/null")
                ss=file("ss")                
                self.folds.append(ss.readlines())
                ss.close()


if __name__=="__main__":
    sequence=sys.argv[1]
    mfold=MFold()
    folds=mfold.fold(sequence," P=20 ")
    for f in folds:
        print f
    rnasubopt=RNAsubopt()
    for f in rnasubopt.fold(sequence," -e 10 -p 10 "):
        print f
    rnasubopt=RNAshapes()
    for f in rnasubopt.fold(sequence):
        print f

